#!/usr/bin/env python
#
# PNG to nametable editor .sav converter using Python Imaging Library
# Copyright 2012 Damian Yerrick
#
# Copying and distribution of this file, with or without
# modification, are permitted in any medium without royalty
# provided the copyright notice and this notice are preserved.
# This file is offered as-is, without any warranty.
#
from __future__ import with_statement
from time import sleep
from PIL import Image
from pilbmp2nes import pilbmp2chr
from chnutils import dedupe_chr

default_palette = '\x0F\x00\x10\x30\x0F\x06\x16\x26\x0F\x1A\x2A\x3A\x0F\x02\x12\x22'

def parse_argv(argv):
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog [options] [-i] INFILE [-o] OUTFILE")
    parser.add_option("-i", "--image", dest="infilename",
                      help="read image from INFILE", metavar="INFILE")
    parser.add_option("-o", "--output", dest="outfilename",
                      help="write SAV data to OUTFILE", metavar="OUTFILE")
    (options, args) = parser.parse_args(argv[1:])

    # Fill unfilled roles with positional arguments
    argsreader = iter(args)
    try:
        infilename = options.infilename
        if infilename is None:
            infilename = argsreader.next()
        outfilename = options.infilename
        if outfilename is None:
            outfilename = argsreader.next()
    except StopIteration:
        raise ValueError("not enough filenames")
    try:
        argsreader.next()
        raise ValueError("too many filenames")
    except StopIteration:
        pass

    return (infilename, outfilename)

argvTestingMode = True

def main(argv=None):
    if argv is None:
        import sys
        argv = sys.argv
        if (argvTestingMode and len(argv) < 2
            and sys.stdin.isatty() and sys.stdout.isatty()):
            argv.extend(raw_input('args:').split())
    try:
        (infilename, outfilename) = parse_argv(argv)
    except StandardError, e:
        import sys
        sys.stderr.write("%s: %s\n" % (argv[0], str(e)))
        sys.exit(1)

    im = Image.open(infilename)
    (w, h) = im.size
    im = pilbmp2chr(im, 8, 8)

    if len(im) <= 256:
        # smaller than 16384 pixels: output as a tile sheet
        # with a blank nametable
        chrdata = ''.join(im)
        namdata = '\xFF' * 960 + '\x00' * 64
    else:
        from array import array
        im, linear_namdata = dedupe_chr(im)
        print len(im), "tiles"
        width_in_tiles = w // 8
        height_in_tiles = len(linear_namdata) // width_in_tiles
        topborder = (31 - height_in_tiles) // 2
        leftborder = (32 - width_in_tiles) // 2
        rightborder = 32 - width_in_tiles - leftborder
        namdata = array('B', '\xFF' * (32 * topborder))
        for i in range(0, len(linear_namdata), width_in_tiles):
            namdata.fromstring('\xFF' * leftborder)
            namdata.extend(linear_namdata[i:i + width_in_tiles])
            namdata.fromstring('\xFF' * rightborder)
        namdata.fromstring('\xFF' * (960 - len(namdata)))
        namdata.fromstring('\x00' * 64)
        namdata = namdata.tostring()
        assert len(namdata) == 1024

    chrdata = ''.join(im)
    chrdata = chrdata + '\xFF' * (4096 - len(chrdata))

    sav = ''.join((chrdata,
                   '\xFF' * 2048,
                   namdata,
                   '\xFF' * 768,
                   default_palette, default_palette,
                   '\xFF' * 224))
    assert len(sav) == 8192
    with open(outfilename, 'wb') as outfp:
        outfp.write(sav)
    

if __name__=='__main__':
    main()
##    main(['png2sav.py', '../docs/kiddie scratch1.png', '/home/pino/.fceux/sav/editor.sav'])
    
